import sys
import os
import asyncio
import subprocess
import argparse
from datetime import datetime, timedelta
import re
import json
import pyte
import time
import threading
import platform
from telegram import Bot
from ._telegram_config import run as telegram_config

parser = argparse.ArgumentParser(description='Telegram notifier')
parser.add_argument('message', nargs='?', default='Notification from {N}', type=str, help='The message to send')
parser.add_argument('-p', '--periodic', nargs=2, metavar=('period', 'command'), help='Execute a command getting periodical updates')
parser.add_argument('-v', '--verbose', action='store_true', help='Verbose mode')
parser.add_argument('-f', '--file', type=str, help='File to send with relative path')
parser.add_argument('--config', action='store_true', help='Config telegram bot')

config_path = os.path.expanduser('~/.config/OpenNTFY/config.json') if platform.system() == 'Linux' else os.path.expanduser('~/AppData/Roaming/OpenNTFY/config.json')

loop = asyncio.get_event_loop()
screen = pyte.Screen(80, 24)
config = {}

def send(message, bot):
    loop.run_until_complete(bot.send_message(chat_id=config['TELEGRAM_CHAT_ID'], text=message))

def parse_screen():
    return '\n'.join(screen.display) if screen.display else ''

def periodic_send(period, bot):
    time.sleep(1)
    while True:
        message_p = parse_screen()
        if message_p:
            send(message_p, bot)
        time.sleep(period.total_seconds())

def parse_time_string(time_str):
    parts = re.findall(r'.*?[smhd]', time_str)
    total_seconds = 0

    for part in parts:
        value = int(part[:-1])
        unit = part[-1]

        if unit == 's':
            total_seconds += value
        elif unit == 'm':
            total_seconds += value * 60
        elif unit == 'h':
            total_seconds += value * 3600
        elif unit == 'd':
            total_seconds += value * 86400
        else:
            raise ValueError(f"Time unit not recognized: {unit}")

    return timedelta(seconds=total_seconds)

def main():
    try:
        args = parser.parse_args()

        if args.config:
            telegram_config(config_path)
            sys.exit(0)

        verboseprint = print if args.verbose else lambda *a: None

        try:
            with open(config_path) as f:
                global config
                config = json.load(f)
        except Exception as e:
            print(f"Error loading config: {str(e)}")
            print("Please run OpenNTFY.py --config")
            sys.exit(1)

        bot = Bot(config['TELEGRAM_TOKEN'])

        dt = datetime.now()
        fields = {
            'N': platform.uname()[1],
            'T': dt.strftime("%H:%M:%S"),
            'D': dt.strftime("%d/%m/ %Y")
        }
        message = args.message.format(**fields) + '\n'

        if args.periodic:
            period = parse_time_string(args.periodic[0])
            command = args.periodic[1]
            verboseprint(f"Periodic execution of {command} every {period}")

            send_thread = threading.Thread(target=lambda: periodic_send(period, bot))
            send_thread.daemon = True
            send_thread.start()

            actual_screen = os.get_terminal_size()
            global screen
            screen = pyte.Screen(actual_screen.columns, actual_screen.lines)
            stream = pyte.Stream(screen)

            process = subprocess.Popen(
                f'stdbuf -oL {command}',
                shell=True,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                universal_newlines=True
            )
            for c in iter(lambda: process.stdout.read(1), ''):
                print(c, end='', flush=True)
                stream.feed(c)

        if not os.isatty(0):
            cmd = sys.stdin.read()
            message += cmd

        if args.file:
            verboseprint(f"Sending file: {args.file}")
            try:
                loop.run_until_complete(bot.send_document(chat_id=config['TELEGRAM_CHAT_ID'], document=open(args.file, 'rb'), caption=message))
                verboseprint("File sent")
            except Exception as e:
                print(f"Error during file sending: {str(e)}")
                sys.exit(1)
        else:
            verboseprint(f"Sending message: {message}")
            try:
                send(message, bot)
                verboseprint("Message sent")
            except Exception as e:
                print(f"Error during message sending: {str(e)}")
                sys.exit(1)

    except KeyboardInterrupt:
        sys.exit(1)

if __name__ == '__main__':
    main()



import asyncio
import json
import os
from textual.app import App, ComposeResult
from textual.widgets import Input, Label, Button
from textual.containers import Center, Horizontal
from telegram import Bot

class TokenSet(App):
    CSS = """
    #token {
        width: 70%;
    }
    #buttons {
        width: 70%;
    }
    #buttons > * {
        width: 50%;
    }
    """
    def compose(self) -> ComposeResult:
        with Center(): yield Label("Insert your Telegram Bot Token")
        with Center(): yield Input(placeholder="Telegram Bot Token", id="token")
        with Center(): yield Label("Do you know your chat id?")
        with Center():
            with Horizontal(id="buttons"):
                yield Button("Yes", id="yes", variant="primary")
                yield Button("No", id="no", variant="error")
    def on_button_pressed(self, event: Button.Pressed) -> None:
        self.exit((self.query_one('#token').value, event.button.id))

class ChatIdSet(App):
    CSS = """
    #chatid {
        width: 70%;
    }
    """
    def compose(self) -> ComposeResult:
        with Center(): yield Label("Insert your chat id")
        with Center(): yield Input(placeholder="Chat id", id="chatid")
        with Center(): yield Button("Confirm", id="confirm", variant="primary")
    def on_button_pressed(self, event: Button.Pressed) -> None:
        self.exit(self.query_one('#chatid').value)
        
class GetChatId(App):
    def compose(self) -> ComposeResult:
        with Center(): yield Label("Send a message to your bot")
        with Center(): yield Label("Press the button below when you're done")
        with Center(): yield Button("Done", id="done", variant="primary")
    def on_button_pressed(self, event: Button.Pressed) -> None:
        self.exit()

class CheckId(App):
    def __init__(self, username: str = "", chat_id: int = 0):
        self.username = username
        self.chat_id = chat_id
        super().__init__()
    CSS = """
    #buttons {
        width: 70%;
    }
    #buttons > * {
        width: 50%;
    }
    """
    def compose(self) -> ComposeResult:
        with Center(): yield Label("You are:")
        with Center(): yield Label("@" + self.username + " chat id: " + str(self.chat_id))
        with Center(): yield Label("Is that you?")
        with Center(): 
            with Horizontal(id="buttons"):
                yield Button("Yes", id="yes", variant="primary")
                yield Button("No", id="no", variant="error")
    def on_button_pressed(self, event: Button.Pressed) -> None:
        self.exit(event.button.id)

class TestBot(App):
    def compose(self) -> ComposeResult:
        with Center(): yield Label("Send a test message to your bot")
        with Center(): yield Button("Send test message", id="send", variant="primary")
    def on_button_pressed(self, event: Button.Pressed) -> None:
        self.exit()

class FinalCheck(App):
    CSS = """
    #buttons {
        width: 70%;
    }
    #buttons > * {
        width: 50%;
    }
    """
    def compose(self) -> ComposeResult:
        with Center(): yield Label("Did you receive the message?")
        with Center(): 
            with Horizontal(id="buttons"):
                yield Button("Yes", id="yes", variant="primary")
                yield Button("No", id="no", variant="error")
    def on_button_pressed(self, event: Button.Pressed) -> None:
        self.exit(event.button.id)

async def run_config(config_path: str):
    finished = False
    while not finished:
        token, known_chatid = await TokenSet().run_async()
        bot = Bot(token)
        if known_chatid == "yes":
            chat_id = await ChatIdSet().run_async()
        else:
            getted_id = False
            while not getted_id:
                await GetChatId().run_async()
                up = await bot.get_updates(offset=-1, limit=1, timeout=0)
                chat_id = up[0]['message']['chat']['id']
                username = up[0]['message']['chat']['username']
                getted_id = True if await CheckId(username=username, chat_id=chat_id).run_async() == "yes" else False

        await TestBot().run_async()
        await bot.send_message(chat_id=chat_id, text="Test message")
        finished = True if await FinalCheck().run_async() == "yes" else False

    if not os.path.exists(config_path):
        os.makedirs(os.path.dirname(config_path), exist_ok=True)
    
    with open(config_path, 'w') as config_file:
        config = {'TELEGRAM_TOKEN': token, 'TELEGRAM_CHAT_ID': chat_id}
        config_file.write(json.dumps(config))

def run(config_path: str):
    asyncio.run(run_config(config_path))

if __name__ == "__main__":
    run()



from setuptools import setup, find_packages

setup(
    name='OpenNTFY',
    version='0.1.1',
    author='Flavio Renzi',
    author_email='flavioinv@gmail.com',
    description='A simple command line tool to send notifications to your telegram bot',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/FlavioRenzi/OpenNTFY',
    packages=find_packages(),
    classifiers=[
        'Programming Language :: Python :: 3',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
    ],
    python_requires='>=3.6',
    entry_points={
        'console_scripts': [
            'OpenNTFY=OpenNTFY.OpenNTFY:main',
        ],
    },
)
